"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Cards(BaseSDK):
    def link(
        self,
        *,
        account_id: str,
        card_number: str,
        card_cvv: str,
        expiration: Union[
            components.CardExpiration, components.CardExpirationTypedDict
        ],
        billing_address: Union[components.CardAddress, components.CardAddressTypedDict],
        x_wait_for: Optional[components.LinkCardWaitFor] = None,
        e2ee: Optional[
            Union[components.E2EEToken, components.E2EETokenTypedDict]
        ] = None,
        holder_name: Optional[str] = None,
        card_on_file: Optional[bool] = None,
        merchant_account_id: Optional[str] = None,
        verify_name: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.LinkCardResponse:
        r"""Link a card to an existing Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/#link-a-card) to learn more.

        Only use this endpoint if you have provided Moov with a copy of your PCI attestation of compliance.

        During card linking, the provided data will be verified by submitting a $0 authorization (account verification) request.
        If `merchantAccountID` is provided, the authorization request will contain that account's statement descriptor and address.
        Otherwise, the platform account's profile will be used. If no statement descriptor has been set, the authorization will
        use the account's name instead.

        It is strongly recommended that callers include the `X-Wait-For` header, set to `payment-method`, if the newly linked
        card is intended to be used right away. If this header is not included, the caller will need to poll the [List Payment
        Methods](https://docs.moov.io/api/sources/payment-methods/list/)
        endpoint to wait for the new payment methods to be available for use.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_number:
        :param card_cvv:
        :param expiration: The expiration date of the card or token.
        :param billing_address:
        :param x_wait_for: Optional header to wait for certain events, such as the creation of a payment method, to occur before returning a response.

            When this header is set to `payment-method`, the response will include any payment methods that were created for the newly
            linked card in the `paymentMethods` field. Otherwise, the `paymentMethods` field will be omitted from the response.
        :param e2ee: Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
            This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
            [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
        :param holder_name:
        :param card_on_file:
        :param merchant_account_id:
        :param verify_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.LinkCardRequest(
            x_wait_for=x_wait_for,
            account_id=account_id,
            link_card=components.LinkCard(
                e2ee=utils.get_pydantic_model(e2ee, Optional[components.E2EEToken]),
                card_number=card_number,
                card_cvv=card_cvv,
                expiration=utils.get_pydantic_model(
                    expiration, components.CardExpiration
                ),
                holder_name=holder_name,
                billing_address=utils.get_pydantic_model(
                    billing_address, components.CardAddress
                ),
                card_on_file=card_on_file,
                merchant_account_id=merchant_account_id,
                verify_name=verify_name,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/cards",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.LinkCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.link_card, False, False, "json", components.LinkCard
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="linkCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.LinkCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                errors.DuplicateCardErrorData, http_res
            )
            raise errors.DuplicateCardError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(errors.LinkCardErrorData, http_res)
            raise errors.LinkCardError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def link_async(
        self,
        *,
        account_id: str,
        card_number: str,
        card_cvv: str,
        expiration: Union[
            components.CardExpiration, components.CardExpirationTypedDict
        ],
        billing_address: Union[components.CardAddress, components.CardAddressTypedDict],
        x_wait_for: Optional[components.LinkCardWaitFor] = None,
        e2ee: Optional[
            Union[components.E2EEToken, components.E2EETokenTypedDict]
        ] = None,
        holder_name: Optional[str] = None,
        card_on_file: Optional[bool] = None,
        merchant_account_id: Optional[str] = None,
        verify_name: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.LinkCardResponse:
        r"""Link a card to an existing Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/#link-a-card) to learn more.

        Only use this endpoint if you have provided Moov with a copy of your PCI attestation of compliance.

        During card linking, the provided data will be verified by submitting a $0 authorization (account verification) request.
        If `merchantAccountID` is provided, the authorization request will contain that account's statement descriptor and address.
        Otherwise, the platform account's profile will be used. If no statement descriptor has been set, the authorization will
        use the account's name instead.

        It is strongly recommended that callers include the `X-Wait-For` header, set to `payment-method`, if the newly linked
        card is intended to be used right away. If this header is not included, the caller will need to poll the [List Payment
        Methods](https://docs.moov.io/api/sources/payment-methods/list/)
        endpoint to wait for the new payment methods to be available for use.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_number:
        :param card_cvv:
        :param expiration: The expiration date of the card or token.
        :param billing_address:
        :param x_wait_for: Optional header to wait for certain events, such as the creation of a payment method, to occur before returning a response.

            When this header is set to `payment-method`, the response will include any payment methods that were created for the newly
            linked card in the `paymentMethods` field. Otherwise, the `paymentMethods` field will be omitted from the response.
        :param e2ee: Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
            This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
            [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
        :param holder_name:
        :param card_on_file:
        :param merchant_account_id:
        :param verify_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.LinkCardRequest(
            x_wait_for=x_wait_for,
            account_id=account_id,
            link_card=components.LinkCard(
                e2ee=utils.get_pydantic_model(e2ee, Optional[components.E2EEToken]),
                card_number=card_number,
                card_cvv=card_cvv,
                expiration=utils.get_pydantic_model(
                    expiration, components.CardExpiration
                ),
                holder_name=holder_name,
                billing_address=utils.get_pydantic_model(
                    billing_address, components.CardAddress
                ),
                card_on_file=card_on_file,
                merchant_account_id=merchant_account_id,
                verify_name=verify_name,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/cards",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.LinkCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.link_card, False, False, "json", components.LinkCard
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="linkCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.LinkCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                errors.DuplicateCardErrorData, http_res
            )
            raise errors.DuplicateCardError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(errors.LinkCardErrorData, http_res)
            raise errors.LinkCardError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListCardsResponse:
        r"""List all the active cards associated with a Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListCardsRequest(
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/cards",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListCardsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listCards",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListCardsResponse(
                result=unmarshal_json_response(List[components.Card], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListCardsResponse:
        r"""List all the active cards associated with a Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListCardsRequest(
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/cards",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListCardsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listCards",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListCardsResponse(
                result=unmarshal_json_response(List[components.Card], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        card_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetCardResponse:
        r"""Fetch a specific card associated with a Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.read` scope.

        :param account_id:
        :param card_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetCardRequest(
            account_id=account_id,
            card_id=card_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        card_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetCardResponse:
        r"""Fetch a specific card associated with a Moov account.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.read` scope.

        :param account_id:
        :param card_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetCardRequest(
            account_id=account_id,
            card_id=card_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        account_id: str,
        card_id: str,
        e2ee: Optional[
            Union[components.E2EETokenUpdate, components.E2EETokenUpdateTypedDict]
        ] = None,
        billing_address: Optional[
            Union[components.UpdateCardAddress, components.UpdateCardAddressTypedDict]
        ] = None,
        expiration: Optional[
            Union[
                components.UpdateCardExpiration,
                components.UpdateCardExpirationTypedDict,
            ]
        ] = None,
        card_cvv: Optional[str] = None,
        card_on_file: Optional[bool] = None,
        merchant_account_id: Optional[str] = None,
        verify_name: Optional[bool] = None,
        holder_name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateCardResponse:
        r"""Update a linked card and/or resubmit it for verification.

        If a value is provided for CVV, a new verification ($0 authorization) will be submitted for the card. Updating the expiration
        date or
        address will update the information stored on file for the card but will not be verified.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/#reverify-a-card) to learn
        more.

        Only use this endpoint if you have provided Moov with a copy of your PCI attestation of compliance.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_id:
        :param e2ee: Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
            This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
            [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
        :param billing_address:
        :param expiration:
        :param card_cvv:
        :param card_on_file:
        :param merchant_account_id:
        :param verify_name:
        :param holder_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateCardRequest(
            account_id=account_id,
            card_id=card_id,
            update_card=components.UpdateCard(
                e2ee=utils.get_pydantic_model(
                    e2ee, Optional[components.E2EETokenUpdate]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[components.UpdateCardAddress]
                ),
                expiration=utils.get_pydantic_model(
                    expiration, Optional[components.UpdateCardExpiration]
                ),
                card_cvv=card_cvv,
                card_on_file=card_on_file,
                merchant_account_id=merchant_account_id,
                verify_name=verify_name,
                holder_name=holder_name,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_card, False, False, "json", components.UpdateCard
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdateCardErrorData, http_res
            )
            raise errors.UpdateCardError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        account_id: str,
        card_id: str,
        e2ee: Optional[
            Union[components.E2EETokenUpdate, components.E2EETokenUpdateTypedDict]
        ] = None,
        billing_address: Optional[
            Union[components.UpdateCardAddress, components.UpdateCardAddressTypedDict]
        ] = None,
        expiration: Optional[
            Union[
                components.UpdateCardExpiration,
                components.UpdateCardExpirationTypedDict,
            ]
        ] = None,
        card_cvv: Optional[str] = None,
        card_on_file: Optional[bool] = None,
        merchant_account_id: Optional[str] = None,
        verify_name: Optional[bool] = None,
        holder_name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateCardResponse:
        r"""Update a linked card and/or resubmit it for verification.

        If a value is provided for CVV, a new verification ($0 authorization) will be submitted for the card. Updating the expiration
        date or
        address will update the information stored on file for the card but will not be verified.

        Read our [accept card payments guide](https://docs.moov.io/guides/sources/cards/accept-card-payments/#reverify-a-card) to learn
        more.

        Only use this endpoint if you have provided Moov with a copy of your PCI attestation of compliance.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_id:
        :param e2ee: Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
            This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
            [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
        :param billing_address:
        :param expiration:
        :param card_cvv:
        :param card_on_file:
        :param merchant_account_id:
        :param verify_name:
        :param holder_name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateCardRequest(
            account_id=account_id,
            card_id=card_id,
            update_card=components.UpdateCard(
                e2ee=utils.get_pydantic_model(
                    e2ee, Optional[components.E2EETokenUpdate]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[components.UpdateCardAddress]
                ),
                expiration=utils.get_pydantic_model(
                    expiration, Optional[components.UpdateCardExpiration]
                ),
                card_cvv=card_cvv,
                card_on_file=card_on_file,
                merchant_account_id=merchant_account_id,
                verify_name=verify_name,
                holder_name=holder_name,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_card, False, False, "json", components.UpdateCard
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateCardResponse(
                result=unmarshal_json_response(components.Card, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdateCardErrorData, http_res
            )
            raise errors.UpdateCardError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def disable(
        self,
        *,
        account_id: str,
        card_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DisableCardResponse:
        r"""Disables a card associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DisableCardRequest(
            account_id=account_id,
            card_id=card_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.DisableCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="disableCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.DisableCardResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def disable_async(
        self,
        *,
        account_id: str,
        card_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DisableCardResponse:
        r"""Disables a card associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id:
        :param card_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DisableCardRequest(
            account_id=account_id,
            card_id=card_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/accounts/{accountID}/cards/{cardID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.DisableCardGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="disableCard",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.DisableCardResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class IssuingIntervalLimit(str, Enum):
    r"""Specifies the time frame for the velocity limit. Currently supports only per-transaction limits."""

    PER_TRANSACTION = "per-transaction"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applepayheader import ApplePayHeader, ApplePayHeaderTypedDict
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class LinkApplePaymentDataTypedDict(TypedDict):
    r"""Contains the encrypted payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken/1916115-paymentdata)
    for more information.
    """

    version: str
    r"""Specifies the encryption used for the payment token (\"EC_v1\" or \"RSA_v1\")."""
    data: str
    r"""The encrypted, base64-encoded payment data."""
    signature: str
    r"""A base64 encoded signature of the payment and header data."""
    header: ApplePayHeaderTypedDict
    r"""Information needed to decrypt Apple Pay payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/passkit/payment-token-format-reference#Header-keys-and-values)
    for more information.
    """


class LinkApplePaymentData(BaseModel):
    r"""Contains the encrypted payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken/1916115-paymentdata)
    for more information.
    """

    version: str
    r"""Specifies the encryption used for the payment token (\"EC_v1\" or \"RSA_v1\")."""

    data: str
    r"""The encrypted, base64-encoded payment data."""

    signature: str
    r"""A base64 encoded signature of the payment and header data."""

    header: ApplePayHeader
    r"""Information needed to decrypt Apple Pay payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/passkit/payment-token-format-reference#Header-keys-and-values)
    for more information.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TransactionSource(str, Enum):
    r"""Specifies the nature and initiator of a transaction.

    Crucial for recurring and merchant-initiated transactions as per card scheme rules.
    Omit for customer-initiated e-commerce transactions.
    """

    FIRST_RECURRING = "first-recurring"
    RECURRING = "recurring"
    UNSCHEDULED = "unscheduled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FeeCategory(str, Enum):
    ACH = "ach"
    CARD_ACQUIRING = "card-acquiring"
    CARD_OTHER = "card-other"
    CARD_PULL = "card-pull"
    CARD_PUSH = "card-push"
    INSTANT_BANK = "instant-bank"
    MONTHLY_PLATFORM = "monthly-platform"
    NETWORK_PASSTHROUGH = "network-passthrough"
    OTHER = "other"
    RTP = "rtp"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class TerminalConfigurationTypedDict(TypedDict):
    r"""Describes a terminal configuration."""

    configuration: str


class TerminalConfiguration(BaseModel):
    r"""Describes a terminal configuration."""

    configuration: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .createpaymentlinklineitems import (
    CreatePaymentLinkLineItems,
    CreatePaymentLinkLineItemsTypedDict,
)
from .paymentlinkcustomeroptions import (
    PaymentLinkCustomerOptions,
    PaymentLinkCustomerOptionsTypedDict,
)
from .paymentlinkdisplayoptions import (
    PaymentLinkDisplayOptions,
    PaymentLinkDisplayOptionsTypedDict,
)
from .paymentlinkpaymentdetails import (
    PaymentLinkPaymentDetails,
    PaymentLinkPaymentDetailsTypedDict,
)
from .paymentlinkpayoutdetails import (
    PaymentLinkPayoutDetails,
    PaymentLinkPayoutDetailsTypedDict,
)
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreatePaymentLinkTypedDict(TypedDict):
    r"""Request to create a new payment link.

    A payment link must include either `payment` or `payout` details, but not both. For payout payment links,
    `maxUses` will automatically be set to 1, as these are intended for a one-time disbursement
    to a specific recipient.

    """

    partner_account_id: str
    r"""The partner's Moov account ID."""
    merchant_payment_method_id: str
    r"""The merchant's preferred payment method ID. Must be a wallet payment method."""
    amount: AmountTypedDict
    display: PaymentLinkDisplayOptionsTypedDict
    r"""Customizable display options for a payment link."""
    sales_tax_amount: NotRequired[AmountTypedDict]
    max_uses: NotRequired[int]
    r"""An optional limit on the number of times this payment link can be used.

    **For payouts, `maxUses` is always 1.**
    """
    expires_on: NotRequired[datetime]
    r"""An optional expiration date for this payment link."""
    customer: NotRequired[PaymentLinkCustomerOptionsTypedDict]
    payment: NotRequired[PaymentLinkPaymentDetailsTypedDict]
    r"""Options for payment links used to collect payment."""
    payout: NotRequired[PaymentLinkPayoutDetailsTypedDict]
    line_items: NotRequired[CreatePaymentLinkLineItemsTypedDict]
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """


class CreatePaymentLink(BaseModel):
    r"""Request to create a new payment link.

    A payment link must include either `payment` or `payout` details, but not both. For payout payment links,
    `maxUses` will automatically be set to 1, as these are intended for a one-time disbursement
    to a specific recipient.

    """

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]
    r"""The partner's Moov account ID."""

    merchant_payment_method_id: Annotated[
        str, pydantic.Field(alias="merchantPaymentMethodID")
    ]
    r"""The merchant's preferred payment method ID. Must be a wallet payment method."""

    amount: Amount

    display: PaymentLinkDisplayOptions
    r"""Customizable display options for a payment link."""

    sales_tax_amount: Annotated[
        Optional[Amount], pydantic.Field(alias="salesTaxAmount")
    ] = None

    max_uses: Annotated[Optional[int], pydantic.Field(alias="maxUses")] = None
    r"""An optional limit on the number of times this payment link can be used.

    **For payouts, `maxUses` is always 1.**
    """

    expires_on: Annotated[Optional[datetime], pydantic.Field(alias="expiresOn")] = None
    r"""An optional expiration date for this payment link."""

    customer: Optional[PaymentLinkCustomerOptions] = None

    payment: Optional[PaymentLinkPaymentDetails] = None
    r"""Options for payment links used to collect payment."""

    payout: Optional[PaymentLinkPayoutDetails] = None

    line_items: Annotated[
        Optional[CreatePaymentLinkLineItems], pydantic.Field(alias="lineItems")
    ] = None
    r"""An optional collection of line items for a payment link.
    When line items are provided, their total plus sales tax must equal the payment link amount.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "salesTaxAmount",
                "maxUses",
                "expiresOn",
                "customer",
                "payment",
                "payout",
                "lineItems",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreatePaymentLink.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional
from typing_extensions import deprecated


class Institutions(BaseSDK):
    def search_institutions(
        self,
        *,
        name: Optional[str] = None,
        routing_number: Optional[str] = None,
        limit: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SearchInstitutionsResponse:
        r"""Search for financial institutions by name or routing number.

        This endpoint returns metadata about each matched institution, including basic identifying details (such as name, routing number, and address) and information about which payment services they support (e.g., ACH, RTP, and Wire).

        This can be used to validate a financial institution before initiating payment activity, or to check which payment rails are available for a given routing number.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/institutions.read` scope.

        :param name: Name of the financial institution. Either `name` or `routingNumber` is required.
        :param routing_number: Routing number for a financial institution. Either `routingNumber` or `name` is required.
        :param limit: Maximum results returned by a search.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SearchInstitutionsRequest(
            name=name,
            routing_number=routing_number,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/institutions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.SearchInstitutionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="searchInstitutions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.SearchInstitutionsResponse(
                result=unmarshal_json_response(
                    components.InstitutionsSearchResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def search_institutions_async(
        self,
        *,
        name: Optional[str] = None,
        routing_number: Optional[str] = None,
        limit: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SearchInstitutionsResponse:
        r"""Search for financial institutions by name or routing number.

        This endpoint returns metadata about each matched institution, including basic identifying details (such as name, routing number, and address) and information about which payment services they support (e.g., ACH, RTP, and Wire).

        This can be used to validate a financial institution before initiating payment activity, or to check which payment rails are available for a given routing number.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/institutions.read` scope.

        :param name: Name of the financial institution. Either `name` or `routingNumber` is required.
        :param routing_number: Routing number for a financial institution. Either `routingNumber` or `name` is required.
        :param limit: Maximum results returned by a search.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SearchInstitutionsRequest(
            name=name,
            routing_number=routing_number,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/institutions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.SearchInstitutionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="searchInstitutions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.SearchInstitutionsResponse(
                result=unmarshal_json_response(
                    components.InstitutionsSearchResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    @deprecated(
        "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
    )
    def search(
        self,
        *,
        name: Optional[str] = None,
        routing_number: Optional[str] = None,
        state: Optional[str] = None,
        limit: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListInstitutionsResponse:
        r"""This endpoint has been deprecated and will be removed in a future release. Use [/institutions](https://docs.moov.io/api/enrichment/form-shortening/institutions/get/).

        Search for institutions by either their name or routing number.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/fed.read` scope.

        :param name: Name of the financial institution. Either `name` or `routingNumber` is required.
        :param routing_number: Routing number for a financial institution. Either `routingNumber` or `name` is required.
        :param state: The state where a financial institution is based.
        :param limit: Maximum results returned by a search.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListInstitutionsRequest(
            name=name,
            routing_number=routing_number,
            state=state,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/institutions/ach/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListInstitutionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listInstitutions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListInstitutionsResponse(
                result=unmarshal_json_response(
                    components.FinancialInstitutions, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    @deprecated(
        "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
    )
    async def search_async(
        self,
        *,
        name: Optional[str] = None,
        routing_number: Optional[str] = None,
        state: Optional[str] = None,
        limit: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListInstitutionsResponse:
        r"""This endpoint has been deprecated and will be removed in a future release. Use [/institutions](https://docs.moov.io/api/enrichment/form-shortening/institutions/get/).

        Search for institutions by either their name or routing number.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/fed.read` scope.

        :param name: Name of the financial institution. Either `name` or `routingNumber` is required.
        :param routing_number: Routing number for a financial institution. Either `routingNumber` or `name` is required.
        :param state: The state where a financial institution is based.
        :param limit: Maximum results returned by a search.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListInstitutionsRequest(
            name=name,
            routing_number=routing_number,
            state=state,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/institutions/ach/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListInstitutionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listInstitutions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListInstitutionsResponse(
                result=unmarshal_json_response(
                    components.FinancialInstitutions, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountexception import BankAccountException, BankAccountExceptionTypedDict
from .bankaccountverificationmethod import BankAccountVerificationMethod
from .bankaccountverificationstatus import BankAccountVerificationStatus
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BankAccountVerificationTypedDict(TypedDict):
    verification_method: BankAccountVerificationMethod
    status: BankAccountVerificationStatus
    exception_details: NotRequired[BankAccountExceptionTypedDict]
    r"""Reason for, and details related to, an `errored` or `verificationFailed` bank account status."""


class BankAccountVerification(BaseModel):
    verification_method: Annotated[
        BankAccountVerificationMethod, pydantic.Field(alias="verificationMethod")
    ]

    status: BankAccountVerificationStatus

    exception_details: Annotated[
        Optional[BankAccountException], pydantic.Field(alias="exceptionDetails")
    ] = None
    r"""Reason for, and details related to, an `errored` or `verificationFailed` bank account status."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["exceptionDetails"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    BankAccountVerification.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookbillingstatementcreated import (
    WebhookBillingStatementCreated,
    WebhookBillingStatementCreatedTypedDict,
)
from .webhookdataaccountcreated import (
    WebhookDataAccountCreated,
    WebhookDataAccountCreatedTypedDict,
)
from .webhookdataaccountdisconnected import (
    WebhookDataAccountDisconnected,
    WebhookDataAccountDisconnectedTypedDict,
)
from .webhookdataaccountupdated import (
    WebhookDataAccountUpdated,
    WebhookDataAccountUpdatedTypedDict,
)
from .webhookdatabalanceupdated import (
    WebhookDataBalanceUpdated,
    WebhookDataBalanceUpdatedTypedDict,
)
from .webhookdatabankaccountcreated import (
    WebhookDataBankAccountCreated,
    WebhookDataBankAccountCreatedTypedDict,
)
from .webhookdatabankaccountdeleted import (
    WebhookDataBankAccountDeleted,
    WebhookDataBankAccountDeletedTypedDict,
)
from .webhookdatabankaccountupdated import (
    WebhookDataBankAccountUpdated,
    WebhookDataBankAccountUpdatedTypedDict,
)
from .webhookdatacancellationcreated import (
    WebhookDataCancellationCreated,
    WebhookDataCancellationCreatedTypedDict,
)
from .webhookdatacancellationupdated import (
    WebhookDataCancellationUpdated,
    WebhookDataCancellationUpdatedTypedDict,
)
from .webhookdatacapabilityrequested import (
    WebhookDataCapabilityRequested,
    WebhookDataCapabilityRequestedTypedDict,
)
from .webhookdatacapabilityupdated import (
    WebhookDataCapabilityUpdated,
    WebhookDataCapabilityUpdatedTypedDict,
)
from .webhookdatacardautoupdated import (
    WebhookDataCardAutoUpdated,
    WebhookDataCardAutoUpdatedTypedDict,
)
from .webhookdatadisputecreated import (
    WebhookDataDisputeCreated,
    WebhookDataDisputeCreatedTypedDict,
)
from .webhookdatadisputeupdated import (
    WebhookDataDisputeUpdated,
    WebhookDataDisputeUpdatedTypedDict,
)
from .webhookdatainvoicecreated import (
    WebhookDataInvoiceCreated,
    WebhookDataInvoiceCreatedTypedDict,
)
from .webhookdatainvoiceupdated import (
    WebhookDataInvoiceUpdated,
    WebhookDataInvoiceUpdatedTypedDict,
)
from .webhookdatanetworkidupdated import (
    WebhookDataNetworkIDUpdated,
    WebhookDataNetworkIDUpdatedTypedDict,
)
from .webhookdatapaymentmethoddisabled import (
    WebhookDataPaymentMethodDisabled,
    WebhookDataPaymentMethodDisabledTypedDict,
)
from .webhookdatapaymentmethodenabled import (
    WebhookDataPaymentMethodEnabled,
    WebhookDataPaymentMethodEnabledTypedDict,
)
from .webhookdatarefundcreated import (
    WebhookDataRefundCreated,
    WebhookDataRefundCreatedTypedDict,
)
from .webhookdatarefundupdated import (
    WebhookDataRefundUpdated,
    WebhookDataRefundUpdatedTypedDict,
)
from .webhookdatarepresentativecreated import (
    WebhookDataRepresentativeCreated,
    WebhookDataRepresentativeCreatedTypedDict,
)
from .webhookdatarepresentativedisabled import (
    WebhookDataRepresentativeDisabled,
    WebhookDataRepresentativeDisabledTypedDict,
)
from .webhookdatarepresentativeupdated import (
    WebhookDataRepresentativeUpdated,
    WebhookDataRepresentativeUpdatedTypedDict,
)
from .webhookdatasweepcreated import (
    WebhookDataSweepCreated,
    WebhookDataSweepCreatedTypedDict,
)
from .webhookdatasweepupdated import (
    WebhookDataSweepUpdated,
    WebhookDataSweepUpdatedTypedDict,
)
from .webhookdataterminalapplicationcreated import (
    WebhookDataTerminalApplicationCreated,
    WebhookDataTerminalApplicationCreatedTypedDict,
)
from .webhookdataterminalapplicationupdated import (
    WebhookDataTerminalApplicationUpdated,
    WebhookDataTerminalApplicationUpdatedTypedDict,
)
from .webhookdataticketcreated import (
    WebhookDataTicketCreated,
    WebhookDataTicketCreatedTypedDict,
)
from .webhookdataticketmessageadded import (
    WebhookDataTicketMessageAdded,
    WebhookDataTicketMessageAddedTypedDict,
)
from .webhookdataticketupdated import (
    WebhookDataTicketUpdated,
    WebhookDataTicketUpdatedTypedDict,
)
from .webhookdatatransfercreated import (
    WebhookDataTransferCreated,
    WebhookDataTransferCreatedTypedDict,
)
from .webhookdatatransferupdated import (
    WebhookDataTransferUpdated,
    WebhookDataTransferUpdatedTypedDict,
)
from .webhookdatawalletcreated import (
    WebhookDataWalletCreated,
    WebhookDataWalletCreatedTypedDict,
)
from .webhookdatawallettransactionupdated import (
    WebhookDataWalletTransactionUpdated,
    WebhookDataWalletTransactionUpdatedTypedDict,
)
from .webhookdatawalletupdated import (
    WebhookDataWalletUpdated,
    WebhookDataWalletUpdatedTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


WebhookDataTypedDict = TypeAliasType(
    "WebhookDataTypedDict",
    Union[
        WebhookBillingStatementCreatedTypedDict,
        WebhookDataBankAccountDeletedTypedDict,
        WebhookDataTerminalApplicationUpdatedTypedDict,
        WebhookDataBalanceUpdatedTypedDict,
        WebhookDataBankAccountCreatedTypedDict,
        WebhookDataAccountUpdatedTypedDict,
        WebhookDataRepresentativeCreatedTypedDict,
        WebhookDataWalletCreatedTypedDict,
        WebhookDataAccountDisconnectedTypedDict,
        WebhookDataAccountCreatedTypedDict,
        WebhookDataTerminalApplicationCreatedTypedDict,
        WebhookDataSweepCreatedTypedDict,
        WebhookDataRepresentativeDisabledTypedDict,
        WebhookDataRepresentativeUpdatedTypedDict,
        WebhookDataInvoiceCreatedTypedDict,
        WebhookDataCardAutoUpdatedTypedDict,
        WebhookDataCapabilityRequestedTypedDict,
        WebhookDataPaymentMethodEnabledTypedDict,
        WebhookDataPaymentMethodDisabledTypedDict,
        WebhookDataRefundCreatedTypedDict,
        WebhookDataWalletUpdatedTypedDict,
        WebhookDataInvoiceUpdatedTypedDict,
        WebhookDataCancellationCreatedTypedDict,
        WebhookDataTicketMessageAddedTypedDict,
        WebhookDataTicketCreatedTypedDict,
        WebhookDataCancellationUpdatedTypedDict,
        WebhookDataSweepUpdatedTypedDict,
        WebhookDataCapabilityUpdatedTypedDict,
        WebhookDataTicketUpdatedTypedDict,
        WebhookDataTransferCreatedTypedDict,
        WebhookDataRefundUpdatedTypedDict,
        WebhookDataWalletTransactionUpdatedTypedDict,
        WebhookDataBankAccountUpdatedTypedDict,
        WebhookDataNetworkIDUpdatedTypedDict,
        WebhookDataDisputeCreatedTypedDict,
        WebhookDataTransferUpdatedTypedDict,
        WebhookDataDisputeUpdatedTypedDict,
    ],
)
r"""The data for the webhook event. The contents are based on the event type."""


WebhookData = TypeAliasType(
    "WebhookData",
    Union[
        WebhookBillingStatementCreated,
        WebhookDataBankAccountDeleted,
        WebhookDataTerminalApplicationUpdated,
        WebhookDataBalanceUpdated,
        WebhookDataBankAccountCreated,
        WebhookDataAccountUpdated,
        WebhookDataRepresentativeCreated,
        WebhookDataWalletCreated,
        WebhookDataAccountDisconnected,
        WebhookDataAccountCreated,
        WebhookDataTerminalApplicationCreated,
        WebhookDataSweepCreated,
        WebhookDataRepresentativeDisabled,
        WebhookDataRepresentativeUpdated,
        WebhookDataInvoiceCreated,
        WebhookDataCardAutoUpdated,
        WebhookDataCapabilityRequested,
        WebhookDataPaymentMethodEnabled,
        WebhookDataPaymentMethodDisabled,
        WebhookDataRefundCreated,
        WebhookDataWalletUpdated,
        WebhookDataInvoiceUpdated,
        WebhookDataCancellationCreated,
        WebhookDataTicketMessageAdded,
        WebhookDataTicketCreated,
        WebhookDataCancellationUpdated,
        WebhookDataSweepUpdated,
        WebhookDataCapabilityUpdated,
        WebhookDataTicketUpdated,
        WebhookDataTransferCreated,
        WebhookDataRefundUpdated,
        WebhookDataWalletTransactionUpdated,
        WebhookDataBankAccountUpdated,
        WebhookDataNetworkIDUpdated,
        WebhookDataDisputeCreated,
        WebhookDataTransferUpdated,
        WebhookDataDisputeUpdated,
    ],
)
r"""The data for the webhook event. The contents are based on the event type."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class TicketMessageTypedDict(TypedDict):
    author: str
    body: str
    sent_on: datetime


class TicketMessage(BaseModel):
    author: str

    body: str

    sent_on: Annotated[datetime, pydantic.Field(alias="sentOn")]


try:
    TicketMessage.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .achexception import ACHException, ACHExceptionTypedDict
from .achtransactionstatus import ACHTransactionStatus
from .debitholdperiod import DebitHoldPeriod
from .seccode import SECCode
from .transferachaddendarecord import (
    TransferACHAddendaRecord,
    TransferACHAddendaRecordTypedDict,
)
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ACHTransactionDetailsTypedDict(TypedDict):
    r"""ACH specific details about the transaction."""

    status: NotRequired[ACHTransactionStatus]
    r"""Status of a transaction within the ACH lifecycle."""
    trace_number: NotRequired[str]
    return_: NotRequired[ACHExceptionTypedDict]
    correction: NotRequired[ACHExceptionTypedDict]
    company_entry_description: NotRequired[str]
    r"""An optional override of the default NACHA company entry description for a transfer."""
    originating_company_name: NotRequired[str]
    r"""An optional override of the default NACHA company name for a transfer."""
    sec_code: NotRequired[SECCode]
    r"""Code used to identify the ACH authorization method."""
    canceled_on: NotRequired[datetime]
    initiated_on: NotRequired[datetime]
    originated_on: NotRequired[datetime]
    corrected_on: NotRequired[datetime]
    returned_on: NotRequired[datetime]
    failed_on: NotRequired[datetime]
    completed_on: NotRequired[datetime]
    debit_hold_period: NotRequired[DebitHoldPeriod]
    r"""An optional override of your default ACH hold period in banking days. The hold period must be longer than or equal to your default setting."""
    addenda: NotRequired[List[TransferACHAddendaRecordTypedDict]]


class ACHTransactionDetails(BaseModel):
    r"""ACH specific details about the transaction."""

    status: Optional[ACHTransactionStatus] = None
    r"""Status of a transaction within the ACH lifecycle."""

    trace_number: Annotated[Optional[str], pydantic.Field(alias="traceNumber")] = None

    return_: Annotated[Optional[ACHException], pydantic.Field(alias="return")] = None

    correction: Optional[ACHException] = None

    company_entry_description: Annotated[
        Optional[str], pydantic.Field(alias="companyEntryDescription")
    ] = None
    r"""An optional override of the default NACHA company entry description for a transfer."""

    originating_company_name: Annotated[
        Optional[str], pydantic.Field(alias="originatingCompanyName")
    ] = None
    r"""An optional override of the default NACHA company name for a transfer."""

    sec_code: Annotated[Optional[SECCode], pydantic.Field(alias="secCode")] = None
    r"""Code used to identify the ACH authorization method."""

    canceled_on: Annotated[Optional[datetime], pydantic.Field(alias="canceledOn")] = (
        None
    )

    initiated_on: Annotated[Optional[datetime], pydantic.Field(alias="initiatedOn")] = (
        None
    )

    originated_on: Annotated[
        Optional[datetime], pydantic.Field(alias="originatedOn")
    ] = None

    corrected_on: Annotated[Optional[datetime], pydantic.Field(alias="correctedOn")] = (
        None
    )

    returned_on: Annotated[Optional[datetime], pydantic.Field(alias="returnedOn")] = (
        None
    )

    failed_on: Annotated[Optional[datetime], pydantic.Field(alias="failedOn")] = None

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    debit_hold_period: Annotated[
        Optional[DebitHoldPeriod], pydantic.Field(alias="debitHoldPeriod")
    ] = None
    r"""An optional override of your default ACH hold period in banking days. The hold period must be longer than or equal to your default setting."""

    addenda: Optional[List[TransferACHAddendaRecord]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "status",
                "traceNumber",
                "return",
                "correction",
                "companyEntryDescription",
                "originatingCompanyName",
                "secCode",
                "canceledOn",
                "initiatedOn",
                "originatedOn",
                "correctedOn",
                "returnedOn",
                "failedOn",
                "completedOn",
                "debitHoldPeriod",
                "addenda",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ACHTransactionDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class IssuingAuthorizationStatus(str, Enum):
    r"""Status of a card issuing authorization."""

    PENDING = "pending"
    DECLINED = "declined"
    CANCELED = "canceled"
    CLEARED = "cleared"
    EXPIRED = "expired"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import importlib.metadata

__title__: str = "moovio_sdk"
__version__: str = "0.0.0-dev.6"
__openapi_doc_version__: str = "dev"
__gen_version__: str = "2.856.1"
__user_agent__: str = "speakeasy-sdk/python 0.0.0-dev.6 2.856.1 dev moovio_sdk"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EstimatedActivityErrorTypedDict(TypedDict):
    average_transaction_amount: NotRequired[str]
    maximum_transaction_amount: NotRequired[str]
    monthly_volume_range: NotRequired[str]


class EstimatedActivityError(BaseModel):
    average_transaction_amount: Annotated[
        Optional[str], pydantic.Field(alias="averageTransactionAmount")
    ] = None

    maximum_transaction_amount: Annotated[
        Optional[str], pydantic.Field(alias="maximumTransactionAmount")
    ] = None

    monthly_volume_range: Annotated[
        Optional[str], pydantic.Field(alias="monthlyVolumeRange")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "averageTransactionAmount",
                "maximumTransactionAmount",
                "monthlyVolumeRange",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    EstimatedActivityError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .productimagemetadata import ProductImageMetadata, ProductImageMetadataTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductOptionTypedDict(TypedDict):
    r"""Represents a single product option within a group."""

    name: str
    r"""The display name of a product option."""
    description: NotRequired[str]
    r"""A detailed description of the option.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """
    price_modifier: NotRequired[AmountDecimalTypedDict]
    r"""The adjustment applied to a product's base price by this option. Can be negative, positive, or zero."""
    images: NotRequired[List[ProductImageMetadataTypedDict]]
    r"""The images associated with this option."""


class ProductOption(BaseModel):
    r"""Represents a single product option within a group."""

    name: str
    r"""The display name of a product option."""

    description: Optional[str] = None
    r"""A detailed description of the option.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """

    price_modifier: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="priceModifier")
    ] = None
    r"""The adjustment applied to a product's base price by this option. Can be negative, positive, or zero."""

    images: Optional[List[ProductImageMetadata]] = None
    r"""The images associated with this option."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "priceModifier", "images"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ProductOption.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .evidencetype import EvidenceType
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class EvidenceTextResponseTypedDict(TypedDict):
    evidence_id: str
    dispute_id: str
    evidence_type: EvidenceType
    text: str
    created_on: datetime


class EvidenceTextResponse(BaseModel):
    evidence_id: Annotated[str, pydantic.Field(alias="evidenceID")]

    dispute_id: Annotated[str, pydantic.Field(alias="disputeID")]

    evidence_type: Annotated[EvidenceType, pydantic.Field(alias="evidenceType")]

    text: str

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    EvidenceTextResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addresserror import AddressError, AddressErrorTypedDict
from .phonenumbererror import PhoneNumberError, PhoneNumberErrorTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateBusinessErrorEinTypedDict(TypedDict):
    number: NotRequired[str]


class CreateBusinessErrorEin(BaseModel):
    number: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["number"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateBusinessErrorTaxIDTypedDict(TypedDict):
    ein: NotRequired[CreateBusinessErrorEinTypedDict]


class CreateBusinessErrorTaxID(BaseModel):
    ein: Optional[CreateBusinessErrorEin] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ein"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateBusinessErrorIndustryCodesTypedDict(TypedDict):
    naics: NotRequired[str]
    sic: NotRequired[str]
    mcc: NotRequired[str]


class CreateBusinessErrorIndustryCodes(BaseModel):
    naics: Optional[str] = None

    sic: Optional[str] = None

    mcc: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["naics", "sic", "mcc"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateBusinessErrorTypedDict(TypedDict):
    legal_business_name: NotRequired[str]
    doing_business_as: NotRequired[str]
    business_type: NotRequired[str]
    address: NotRequired[AddressErrorTypedDict]
    phone: NotRequired[PhoneNumberErrorTypedDict]
    email: NotRequired[str]
    website: NotRequired[str]
    description: NotRequired[str]
    tax_id: NotRequired[CreateBusinessErrorTaxIDTypedDict]
    industry_codes: NotRequired[CreateBusinessErrorIndustryCodesTypedDict]
    industry: NotRequired[str]
    r"""Classification identifier for the industry. Use the [GET industries](https://docs.moov.io/api/enrichment/form-shortening/industries/get/) endpoint to retrieve an array of valid industry details for a merchant, inducing all industry field values."""
    primary_regulator: NotRequired[str]


class CreateBusinessError(BaseModel):
    legal_business_name: Annotated[
        Optional[str], pydantic.Field(alias="legalBusinessName")
    ] = None

    doing_business_as: Annotated[
        Optional[str], pydantic.Field(alias="doingBusinessAs")
    ] = None

    business_type: Annotated[Optional[str], pydantic.Field(alias="businessType")] = None

    address: Optional[AddressError] = None

    phone: Optional[PhoneNumberError] = None

    email: Optional[str] = None

    website: Optional[str] = None

    description: Optional[str] = None

    tax_id: Annotated[
        Optional[CreateBusinessErrorTaxID], pydantic.Field(alias="taxID")
    ] = None

    industry_codes: Annotated[
        Optional[CreateBusinessErrorIndustryCodes],
        pydantic.Field(alias="industryCodes"),
    ] = None

    industry: Optional[str] = None
    r"""Classification identifier for the industry. Use the [GET industries](https://docs.moov.io/api/enrichment/form-shortening/industries/get/) endpoint to retrieve an array of valid industry details for a merchant, inducing all industry field values."""

    primary_regulator: Annotated[
        Optional[str], pydantic.Field(alias="primaryRegulator")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "legalBusinessName",
                "doingBusinessAs",
                "businessType",
                "address",
                "phone",
                "email",
                "website",
                "description",
                "taxID",
                "industryCodes",
                "industry",
                "primaryRegulator",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateBusinessError.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CardPaymentTypedDict(TypedDict):
    statement_descriptor: NotRequired[str]


class CardPayment(BaseModel):
    statement_descriptor: Annotated[
        Optional[str], pydantic.Field(alias="statementDescriptor")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["statementDescriptor"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class AchPaymentTypedDict(TypedDict):
    company_name: NotRequired[str]


class AchPayment(BaseModel):
    company_name: Annotated[Optional[str], pydantic.Field(alias="companyName")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["companyName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateAccountSettingsTypedDict(TypedDict):
    card_payment: NotRequired[CardPaymentTypedDict]
    ach_payment: NotRequired[AchPaymentTypedDict]


class CreateAccountSettings(BaseModel):
    card_payment: Annotated[
        Optional[CardPayment], pydantic.Field(alias="cardPayment")
    ] = None

    ach_payment: Annotated[Optional[AchPayment], pydantic.Field(alias="achPayment")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["cardPayment", "achPayment"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CardPayment.model_rebuild()
except NameError:
    pass
try:
    AchPayment.model_rebuild()
except NameError:
    pass
try:
    CreateAccountSettings.model_rebuild()
except NameError:
    pass
